{*******************************************************}
{                                                       }
{                     Avrora  SOL                       }
{                                                       }
{             Written by Avrora LLC  1999               }
{                                                       }
{*******************************************************}
{ This is a full featured game. Manual written          }
{ code < 250 lines.                                     }
{*******************************************************}
{ You can freely modify and distribute this program.    }
{*******************************************************}

unit UMainForm;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  ESCard, ESGraphics, Menus, ESGameDlg, StdCtrls, Placemnt;

type
  TCardInfo = record
    Value : Integer;
    Suit : TESSuitType;
    CardColor : TESCardColor;
  end;

type
  TMainForm = class(TForm)
    Col1: TESCardGroup;
    Col2: TESCardGroup;
    Col3: TESCardGroup;
    Col4: TESCardGroup;
    Col5: TESCardGroup;
    Col6: TESCardGroup;
    Col7: TESCardGroup;
    Roster1: TESCardGroup;
    Roster2: TESCardGroup;
    Roster3: TESCardGroup;
    Roster4: TESCardGroup;
    Deck: TESCardDeck;
    Table: TESCardGroup;
    DecksDialog: TESDecksDialog;
    Sprite: TESSimpleSprite;
    MainMenu: TMainMenu;
    MenuGame: TMenuItem;
    MenuStart: TMenuItem;
    MenuSep1: TMenuItem;
    MenuDealAnimation: TMenuItem;
    MenuStrictRules: TMenuItem;
    MenuSep2: TMenuItem;
    MenuDecks: TMenuItem;
    MenuSep3: TMenuItem;
    MenuExit: TMenuItem;
    MenuHelp: TMenuItem;
    FormStorage: TFormStorage;
    MenuAutoPlay: TMenuItem;
    MenuContents: TMenuItem;
    N1: TMenuItem;
    About1: TMenuItem;
    MenuDragAnimation: TMenuItem;
    procedure Col7MouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure Col7DragOver(Sender, Source: TObject; X, Y: Integer;
      State: TDragState; var Accept: Boolean);
    procedure Col7DragDrop(Sender, Source: TObject; X, Y: Integer);
    procedure Col7EndDrag(Sender, Target: TObject; X, Y: Integer);
    procedure Col7MouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure Roster4AfterUpdate(Sender: TESCardGroup);
    procedure DecksDialogDeckChange(Sender: TObject;
      BackType: TESBackType);
    procedure DeckClick(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure MenuGameClick(Sender: TObject);
    procedure MenuDecksClick(Sender: TObject);
    procedure MenuStartClick(Sender: TObject);
    procedure MenuStrictRulesClick(Sender: TObject);
    procedure MenuDealAnimationClick(Sender: TObject);
    procedure MenuExitClick(Sender: TObject);
    procedure MenuAboutClick(Sender: TObject);
    procedure Col7Click(Sender: TObject);
    procedure MenuContentsClick(Sender: TObject);
    procedure MenuDragAnimationClick(Sender: TObject);
  private
    { Private declarations }
    Pos : Integer;
    CardCount : Integer;
    CardInfo : TCardInfo;
    procedure NewGame;
    function RuleCheck(Sender : TESCardGroup) : Boolean;
    function RosterRuleCheck(Sender : TESCardGroup) : Boolean;
    procedure Finish;
    procedure BackChange(Sender : TObject);
    procedure SendToRoster(Sender : TObject; Roster : TESCardGroup);
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation

uses UDialogAbout, UHelpForm;

{$R *.DFM}

procedure TMainForm.MenuDecksClick(Sender: TObject);
begin
  DecksDialog.Execute;
end;

procedure TMainForm.DecksDialogDeckChange(Sender: TObject;
  BackType: TESBackType);
begin
  if Sprite.Busy then
    Sprite.AfterMove := BackChange
  else
    BackChange(Self);
end;

procedure TMainForm.Col7MouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if Deck.Busy then
    Exit;
  if Button = mbLeft then
  begin
    Pos := (Sender as TESCardGroup).FindMousePos(X, Y);
    if Pos >= 0 then
    begin
      if (Sender as TControl).Tag <> 2 then
        CardCount := (Sender as TESCardGroup).CardsCount - Pos
      else
        CardCount := 1;
      CardInfo.Suit := (Sender as TESCardGroup)[Pos].Suit;
      CardInfo.Value := (Sender as TESCardGroup)[Pos].Value;
      CardInfo.CardColor := (Sender as TESCardGroup)[Pos].CardColor;
      case (Sender as TESCardGroup).Tag of
         4, 3 : if not (Sender as TESCardGroup)[Pos].ShowBack then
                 (Sender as TESCardGroup).BeginDrag(False, 10);
         2 : if (Pos = (Sender as TESCardGroup).CardsCount - 1) then
                 (Sender as TESCardGroup).BeginDrag(False, 10);
      end;
    end;
  end;
end;

procedure TMainForm.Col7DragOver(Sender, Source: TObject; X,
  Y: Integer; State: TDragState; var Accept: Boolean);
begin
  if (Source is TESCardGroup) and (Sender <> Source) then
    case (Sender as TControl).Tag of
      3 : if MenuStrictRules.Checked then
            Accept := RosterRuleCheck(Sender as TESCardGroup)
          else
            Accept := True;
      2 : Accept := False;
      4 : if MenuStrictRules.Checked then
            Accept := RuleCheck(Sender as TESCardGroup)
          else
            Accept := True;
    end
  else
    Accept := False;
end;

procedure TMainForm.Col7DragDrop(Sender, Source: TObject; X, Y: Integer);
var
  I : Integer;
  L : Integer;
begin
  if Pos >= 0 then
  begin
    case TESCardGroup(Source).Tag of
      4, 3, 2 :
      begin
        L := (Source as TESCardGroup).CardsCount - 1;
        for I := Pos to L do
          with (Sender as TESCardGroup).CardList.Add do
          begin
            //EXPERIMENTAL - START
            if MenuDragAnimation.Checked then
              Sprite.Move((Source as TESCardGroup)[I].Left, (Source as TESCardGroup)[I].Top,
                          (Sender as TESCardGroup).LastCard.Left, (Sender as TESCardGroup).LastCard.Top,
                          (Source as TESCardGroup)[I].FaceBitmap, True);
            //EXPERIMENTAL - FINISH

            Card.ID := (Source as TESCardGroup)[I].ID;
            Card.ShowBack := (Source as TESCardGroup)[I].ShowBack;
          end;

        while (Source as TESCardGroup).CardList.Count - 1 >=  Pos do
          (Source as TESCardGroup).CardList[(Source as TESCardGroup).CardList.Count - 1].Free;
      end;
    end;
  end;
end;

procedure TMainForm.MenuStartClick(Sender: TObject);
begin
  NewGame;
end;

procedure TMainForm.DeckClick(Sender: TObject);
begin
  if Deck.DeckEmpty then
  begin
    Deck.SetDeckTo(Table);
    Table.Reset;
  end
  else
    Deck.Deal(Table, True, 3);
end;

procedure TMainForm.Col7EndDrag(Sender, Target: TObject; X, Y: Integer);
begin
  Pos := -1;
end;

procedure TMainForm.NewGame;
begin
  if Sprite.Busy then
  begin
    Sprite.Stop;
    Exit;
  end;
  if not Deck.Busy then
  begin
    Sprite.AutoErase := True;
    Deck.PrepareDeck;
    if Roster1.CardsCount <> 1 then
      Roster1.Reset;
    if Roster2.CardsCount <> 1 then
      Roster2.Reset;
    if Roster3.CardsCount <> 1 then
      Roster3.Reset;
    if Roster4.CardsCount <> 1 then
      Roster4.Reset;
    Table.Reset;
    Col1.Reset(True);
    Col2.Reset(True);
    Col3.Reset(True);
    Col4.Reset(True);
    Col5.Reset(True);
    Col6.Reset(True);
    Col7.Reset(True);
    Table.Clear;
    Deck.Deal(Col1);
    Deck.Deal(Col2);
    Deck.Deal(Col3);
    Deck.Deal(Col4);
    Deck.Deal(Col5);
    Deck.Deal(Col6);
    Deck.Deal(Col7);
  end;
end;

function TMainForm.RuleCheck(Sender : TESCardGroup) : Boolean;
begin
  Result := False;
  if (Sender.CardsCount = 0) then
  begin
    if (CardInfo.Value = 13) then
      Result := True;
  end
  else
  if (CardInfo.Value = Sender[Sender.CardsCount - 1].PrevValue) and
     (CardInfo.CardColor = Sender[Sender.CardsCount - 1].NextColor) then
     Result := True;
end;

function TMainForm.RosterRuleCheck(Sender: TESCardGroup): Boolean;
begin
  Result := True;
  if CardCount <> 1 then
    Result := False
  else
  if (CardInfo.Suit <> Sender[Sender.CardsCount - 1].Suit) and (Sender.CardsCount > 1) then
    Result := False
  else
  if (Sender.CardsCount = 1) and (CardInfo.Value <> 14) then
    Result := False
  else
  if (Sender.CardsCount > 1) and (CardInfo.Value <> Sender.CardsCount) then
    Result := False
end;

procedure TMainForm.FormActivate(Sender: TObject);
begin
  Update;
  NewGame;
end;

procedure TMainForm.MenuStrictRulesClick(Sender: TObject);
begin
  (Sender as TMenuItem).Checked := not (Sender as TMenuItem).Checked;
end;

procedure TMainForm.MenuDealAnimationClick(Sender: TObject);
begin
  (Sender as TMenuItem).Checked := not (Sender as TMenuItem).Checked;
  Deck.DealAnimation := (Sender as TMenuItem).Checked;
end;

procedure TMainForm.MenuGameClick(Sender: TObject);
begin
  if Sprite.Busy then
    MenuStart.Caption := '&Stop'
  else
    MenuStart.Caption := '&Start'
end;

procedure TMainForm.Finish;
begin
  Sprite.AutoErase := False;
  try
    repeat
      if Roster1.CardsCount > 1 then
        Sprite.Move(Roster1.Left, Roster1.Top, Random(Height), Random(Width),
          Roster1[1].FaceBitmap, True);
      if Roster2.CardsCount > 1 then
        Sprite.Move(Roster2.Left, Roster2.Top, Random(Height), Random(Width),
          Roster2[1].FaceBitmap, True);
      if Roster3.CardsCount > 1 then
        Sprite.Move(Roster3.Left, Roster3.Top, Random(Height), Random(Width),
          Roster3[1].FaceBitmap, True);
      if Roster4.CardsCount > 1 then
        Sprite.Move(Roster4.Left, Roster4.Top, Random(Height), Random(Width),
          Roster4[1].FaceBitmap, True);
     until Application.Terminated;
   finally
     Invalidate;
   end;
end;

procedure TMainForm.MenuExitClick(Sender: TObject);
begin
  Close;
end;

procedure TMainForm.BackChange(Sender: TObject);
var
  I : Integer;
begin
  Sprite.AfterMove := nil;
  for I := 0 to ControlCount - 1 do
    if Controls[I] is TESCustomCardControl then
      if (Controls[I] as TESCustomCardControl).CardBack <> cbDots then
        (Controls[I] as TESCustomCardControl).CardBack := DecksDialog.CardBack;
end;

procedure TMainForm.Roster4AfterUpdate(Sender: TESCardGroup);
begin
  if Roster1.CardsCount + Roster3.CardsCount +
     Roster2.CardsCount + Roster4.CardsCount = 56 then
     Finish;
end;

procedure TMainForm.MenuAboutClick(Sender: TObject);
begin
  Application.ProcessMessages;
  with TDialogAbout.Create(Application) do
  try
    ShowModal;
  finally
    Free;
  end;
end;

procedure TMainForm.SendToRoster(Sender : TObject; Roster: TESCardGroup);
var
  CID : Integer;
  L, T : Integer;
begin
  (Sender as TESCardGroup).LastCard.Empty := True;
  Roster.BeginUpdate;
  try
    CID := (Sender as TESCardGroup).LastCard.ID;
    L := (Sender as TESCardGroup).LastCard.Left;
    T := (Sender as TESCardGroup).LastCard.Top;
    Sprite.Move(L, T, Roster.Left, Roster.Top, (Sender as TESCardGroup).LastCard.FaceBitmap);
    (Sender as TESCardGroup).DeleteLastCard;
    with Roster.CardList.Add do
    begin
      Card.ID := CID;
      Card.ShowBack := False;
    end;
  finally
    Roster.EndUpdate;
    Roster.Redraw;
  end;
end;

procedure TMainForm.Col7MouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  if Deck.Busy then
    Exit;
  if Button = mbRight then
  begin
    if (Sender as TESCardGroup).LastCard = nil then
      Exit;
    if not (Sender as TESCardGroup).LastCard.ShowBack then
    begin
      if (Sender as TESCardGroup).LastCard.Value = 14 then
      begin
        if Roster1.LastCard.ShowBack then
          SendToRoster(Sender, Roster1)
        else
        if Roster2.LastCard.ShowBack then
          SendToRoster(Sender, Roster2)
        else
        if Roster3.LastCard.ShowBack then
          SendToRoster(Sender, Roster3)
        else
        if Roster4.LastCard.ShowBack then
          SendToRoster(Sender, Roster4)
      end
      else
      begin
        if ((Sender as TESCardGroup).LastCard.Value = Roster1.LastCard.NextValue) and
           ((Sender as TESCardGroup).LastCard.Suit = Roster1.LastCard.Suit) and (not Roster1.LastCard.ShowBack) then
           SendToRoster(Sender, Roster1)
        else
        if ((Sender as TESCardGroup).LastCard.Value = Roster2.LastCard.NextValue) and
           ((Sender as TESCardGroup).LastCard.Suit = Roster2.LastCard.Suit) and (not Roster2.LastCard.ShowBack) then
           SendToRoster(Sender, Roster2)
        else
        if ((Sender as TESCardGroup).LastCard.Value = Roster3.LastCard.NextValue) and
           ((Sender as TESCardGroup).LastCard.Suit = Roster3.LastCard.Suit) and (not Roster3.LastCard.ShowBack) then
            SendToRoster(Sender, Roster3)
        else
        if ((Sender as TESCardGroup).LastCard.Value = Roster4.LastCard.NextValue) and
           ((Sender as TESCardGroup).LastCard.Suit = Roster4.LastCard.Suit) and (not Roster4.LastCard.ShowBack) then
           SendToRoster(Sender, Roster4)
      end;
    end;
  end;
end;

procedure TMainForm.Col7Click(Sender: TObject);
begin
  if (Sender as TESCardGroup).CardsCount > 0 then
   if (Sender as TESCardGroup)[(Sender as TESCardGroup).CardsCount - 1].ShowBack then
     (Sender as TESCardGroup)[(Sender as TESCardGroup).CardsCount - 1].ShowBack := False;
end;

procedure TMainForm.MenuContentsClick(Sender: TObject);
begin
  Application.ProcessMessages;
  with THelpForm.Create(Application) do
  try
    ShowModal;
  finally
    Free;
  end;
end;

procedure TMainForm.MenuDragAnimationClick(Sender: TObject);
begin
  (Sender as TMenuItem).Checked := not (Sender as TMenuItem).Checked;
end;

end.


