{*******************************************************}
{                                                       }
{         Avrora Card Gaming Components Demo            }
{                                                       }
{             Written by Avrora LLC  1999               }
{                                                       }
{*******************************************************}
{ You can freely modify and distribute this program.    }
{*******************************************************}


unit UMainForm;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ESCard, ESGraphics, Menus, ESGameDlg, Buttons;

type
  TMainForm = class(TForm)
    ESCardDeck1: TESCardDeck;
    Table: TESCardTable;
    Comp2: TESCardGroup;
    Player: TESCardGroup;
    Comp3: TESCardGroup;
    Comp1: TESCardGroup;
    DecksDialog: TESDecksDialog;
    ScoreDialog: TESScoreDialog;
    Sprite: TESSimpleSprite;
    SpeedButton1: TSpeedButton;
    btFinish: TSpeedButton;
    MainMenu: TMainMenu;
    MenuGame: TMenuItem;
    MenuStart: TMenuItem;
    MenuAutoPlay: TMenuItem;
    MenuSep1: TMenuItem;
    MenuDealAnimation: TMenuItem;
    MenuShowAll: TMenuItem;
    MenuSep2: TMenuItem;
    MenuScore: TMenuItem;
    MenuDecks: TMenuItem;
    MenuSep3: TMenuItem;
    MenuExit: TMenuItem;
    procedure ESCardDeck1CardClick(Sender: TESCardDeck;
      var TakeIt: Boolean);
    procedure TableCardClick(Sender: TESCardTable;
      TopCard: TESSinglecard; var TakeIt: Boolean);
    procedure MenuStartClick(Sender: TObject);
    procedure PlayerCardClick(Sender: TESCardGroup; Card: TESSinglecard;
      Position: Integer; var PlayIt: Boolean; var TablePos: TESTablePos);
    procedure ComputerBeforePlay(Sender: TESCardGroup; var Position: Integer;
      var TablePos: TESTablePos);
    procedure TableAfterDealingFinish(Sender: TESCardTable);
    procedure MenuDecksClick(Sender: TObject);
    procedure MenuScoreClick(Sender: TObject);
    procedure MenuGameClick(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure MenuAutoPlayClick(Sender: TObject);
    procedure SpeedButton1Click(Sender: TObject);
    procedure PlayerMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure PlayerDragOver(Sender, Source: TObject; X, Y: Integer;
      State: TDragState; var Accept: Boolean);
    procedure PlayerDragDrop(Sender, Source: TObject; X, Y: Integer);
    procedure MenuExitClick(Sender: TObject);
    procedure MenuShowAllClick(Sender: TObject);
  private
    { Private declarations }
    C : Integer;
    procedure NewGame;
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation

{$R *.DFM}

procedure TMainForm.NewGame;
begin
  if Sprite.Busy then
    Sprite.Stop;
  if not Table.Busy then
  begin
    Sprite.IncrementX := 10;
    Sprite.IncrementY := 10;
    Table.FinishScoreDelta := 0;
    ScoreDialog.Clear;
    Table.Clear(True);
    Table.PrepareDeck;
    Table.NextCardPos := 1;
    Table.DealToTable;
    Table.Deal(True);
  end;
end;

procedure TMainForm.ESCardDeck1CardClick(Sender: TESCardDeck;
  var TakeIt: Boolean);
begin
  if (Player.NonEmptyCardsCount = 14) then
    TakeIt := True
  else
    ShowMessage('You should play a card.');
end;

procedure TMainForm.TableCardClick(Sender: TESCardTable;
  TopCard: TESSinglecard; var TakeIt: Boolean);
begin
  if (Player.NonEmptyCardsCount = 14) then
    TakeIt := True
  else
    ShowMessage('You should play a card.');
end;

procedure TMainForm.MenuStartClick(Sender: TObject);
begin
  NewGame;
end;

procedure TMainForm.PlayerCardClick(Sender: TESCardGroup;
  Card: TESSinglecard; Position: Integer; var PlayIt: Boolean;
  var TablePos: TESTablePos);
begin
  if (Player.NonEmptyCardsCount = 15) then
    PlayIt := True
  else
    ShowMessage('You should draw a card from the deck or take a card from the table.');
end;

procedure TMainForm.ComputerBeforePlay(Sender: TESCardGroup;
  var Position: Integer; var TablePos: TESTablePos);
begin
  Randomize;
  if Random(8) = 0 then
    Table.SendTopCardTo(Sender)
  else
    Table.SendCardFromDeckTo(Sender);
end;

procedure TMainForm.TableAfterDealingFinish(Sender: TESCardTable);
begin
  Sprite.IncrementX := 3;
  Sprite.IncrementY := 3;
end;

procedure TMainForm.MenuDecksClick(Sender: TObject);
begin
  DecksDialog.Execute;
end;

procedure TMainForm.MenuScoreClick(Sender: TObject);
begin
  ScoreDialog.Execute;
end;

procedure TMainForm.MenuGameClick(Sender: TObject);
begin
  if Sprite.Busy then
    MenuStart.Caption := '&Stop'
  else
    MenuStart.Caption := '&Start'
end;

procedure TMainForm.FormActivate(Sender: TObject);
begin
  Update;
  NewGame;
end;

procedure TMainForm.MenuAutoPlayClick(Sender: TObject);
begin
  TMenuItem(Sender).Checked := not TMenuItem(Sender).Checked;
  if TMenuItem(Sender).Checked then
  begin
    Player.BeforePlay := ComputerBeforePlay;
    Player.Controller := ctComputer;
    Table.StartPlay;
  end
  else
  begin
    Player.Controller := ctHuman;
    Player.BeforePlay := nil;
  end;
end;

procedure TMainForm.SpeedButton1Click(Sender: TObject);
begin
  Player.Sort(soValue);
end;

procedure TMainForm.PlayerMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  if ssRight in Shift then
  begin
    Player.BeginDrag(false, 10);
    C := Player.FindMousePos(X, Y);
  end;
end;

procedure TMainForm.PlayerDragOver(Sender, Source: TObject; X, Y: Integer;
  State: TDragState; var Accept: Boolean);
begin
  if Source is TESCardGroup then
    Accept := True;
end;

procedure TMainForm.PlayerDragDrop(Sender, Source: TObject; X, Y: Integer);
begin
  Player.InsertPos(Player.FindMousePos(X, Y), C);
end;

procedure TMainForm.MenuExitClick(Sender: TObject);
begin
  Close;
end;

procedure TMainForm.MenuShowAllClick(Sender: TObject);
begin
  Comp1.ShowBack := TMenuItem(Sender).Checked;
  Comp2.ShowBack := Comp1.ShowBack;
  Comp3.ShowBack := Comp1.ShowBack;
  TMenuItem(Sender).Checked := not TMenuItem(Sender).Checked;
end;

end.
