{*******************************************************}
{                                                       }
{            Avrora Card Gaming Components              }
{                                                       }
{            Copyright (c) 1999 Avrora LLC              }
{                                                       }
{*******************************************************}
{ This is PROPRIETARY SOURCE CODE of Avrora LLC;        }
{ the contents of this file may not be disclosed to     }
{ third parties, copied or duplicated in any form, in   }
{ whole or in part, without the prior written           }
{ permission of Avrora LLC.                             }
{*******************************************************}

unit ESGameDlg;

interface

uses
  ESCard, Windows, Classes, Forms, Dialogs;

type

  { TYPES }

  TESScoreRecord = record
    Player1 : Integer;
    Player2 : Integer;
    Player3 : Integer;
    Player4 : Integer;
  end;

  TESScoreList = Array of TESScoreRecord;


  { EVENT PROCEDURE DECLERATIONS }

  TDeckChangeEvent = procedure (Sender : TObject; BackType : TESBackType) of object;


  { DIALOG WRAPPER COMPONENTS } {----------------------------------------------}

  TESDecksDialog = class(TComponent)
  private
    FCardBack: TESBackType;
    FOnBackChange: TDeckChangeEvent;
    procedure SetCardBack(const Value: TESBackType);
  public
    function Execute : Boolean;
  published
    property CardBack : TESBackType read FCardBack write SetCardBack;
    property OnDeckChange : TDeckChangeEvent read FOnBackChange write FOnBackChange;
  end;

  {----------------------------------------------------------------------------}

  TESScoreDialog = class(TComponent)
  private
    FPlayer1: TESCardGroup;
    FPlayer3: TESCardGroup;
    FPlayer2: TESCardGroup;
    FPlayer4: TESCardGroup;
    FScoreList : TESScoreList;
    FHighlightLowest: Boolean;
    FCloseDelay: Integer;
    procedure SetCloseDelay(const Value: Integer);
  public
    constructor Create (AComponent : TComponent); override;
    destructor Destroy; override;
    procedure AddScore(ScoreList : array of Integer);
    procedure Clear;
    procedure Update;
    function Execute : Boolean;
  published
    property Player1 : TESCardGroup read FPlayer1 write FPlayer1 default nil;
    property Player2 : TESCardGroup read FPlayer2 write FPlayer2 default nil;
    property Player3 : TESCardGroup read FPlayer3 write FPlayer3 default nil;
    property Player4 : TESCardGroup read FPlayer4 write FPlayer4 default nil;
    property HighlightLowest : Boolean read FHighlightLowest write FHighlightLowest default False;
    property CloseDelay : Integer read FCloseDelay write SetCloseDelay default 0;
  end;

  {----------------------------------------------------------------------------}

implementation

uses
  UDecksDialog, UScoreDialog, Controls;

{ TESDecksDialog }

function TESDecksDialog.Execute: Boolean;
begin
  DecksDialog := TDecksDialog.CreateDeckDialog(Application, FCardBack);
  with DecksDialog do
  try
    ShowModal;
    if ModalResult = mrOk then
      CardBack := SelectedCard;
  finally
    Free;
  end;
  Result := True;
end;

procedure TESDecksDialog.SetCardBack(const Value: TESBackType);
var
  I : Integer;
begin
  if FCardBack <> Value then
  begin
    FCardBack := Value;

    { FIND TABLE DAILOG AND CHANGE ALSO }
    for I := 0 to TForm(Owner).ControlCount - 1 do
      if TForm(Owner).Controls[I] is TESCardTable then
      begin
        if (TForm(Owner).Controls[I] as TESCardTable).CardBack <> Value then
          (TForm(Owner).Controls[I] as TESCardTable).CardBack :=  Value;
        Break;
      end;

    if Assigned(FOnBackChange) then
      FOnBackChange(Self, Value);
  end;
end;

{ TESScoreDialog }

procedure TESScoreDialog.AddScore(ScoreList: array of Integer);
var
  A : Array [0..3] of Integer;
  I : Integer;
begin
  for I := 0 to 3 do
    A[I] := -1;
  if Length(ScoreList) <= 4 then
  begin
    for I := 0 to Length(ScoreList) - 1 do
      A[I] := ScoreList[I];
    SetLength(FScoreList, Length(FScoreList) + 1);
    FScoreList[Length(FScoreList) - 1].Player1 := A[0];
    FScoreList[Length(FScoreList) - 1].Player2 := A[1];
    FScoreList[Length(FScoreList) - 1].Player3 := A[2];
    FScoreList[Length(FScoreList) - 1].Player4 := A[3];
  end;
end;

procedure TESScoreDialog.Clear;
begin
  SetLength(FScoreList, 0);
  if Assigned(Player1) then
  begin
    Player1.Score := 0;
    Player1.ClearTakenCards;
  end;
  if Assigned(Player2) then
  begin
    Player2.Score := 0;
    Player2.ClearTakenCards;
  end;
  if Assigned(Player3) then
  begin
    Player3.Score := 0;
    Player3.ClearTakenCards;
  end;
  if Assigned(Player4) then
  begin  
    Player4.Score := 0;
    Player4.ClearTakenCards;
  end;
end;

constructor TESScoreDialog.Create(AComponent: TComponent);
begin
  inherited;
  SetLength (FScoreList, 0);
end;

destructor TESScoreDialog.Destroy;
begin
  FScoreList := nil;
  inherited Destroy;
end;

function TESScoreDialog.Execute: Boolean;
begin
  ScoreDialog := TScoreDialog.Create(Application);
  with ScoreDialog do
  try
    HighLightLowest := Self.HighLightLowest;
    CloseDelay := Self.CloseDelay;
    if Self.Player1 <> nil then
      Player1 := Self.Player1.PlayerName
    else
      Player1 := '';
    if Self.Player2 <> nil then
      Player2 := Self.Player2.PlayerName
    else
      Player2 := '';
    if Self.Player3 <> nil then
      Player3 := Self.Player3.PlayerName
    else
      Player3 := '';
    if Self.Player4 <> nil then
      Player4 := Self.Player4.PlayerName
    else
      Player4 := '';

    if Length(Self.FScoreList) > 0 then
      FScoreList := Self.FScoreList;
    ShowModal;
  finally
    Free;
  end;
  Result := True;
end;

procedure TESScoreDialog.SetCloseDelay(const Value: Integer);
begin
  if Value >=0 then
  FCloseDelay := Value;
end;

procedure TESScoreDialog.Update;
begin
  if (Player1 <> nil) and (Player2 <> nil) and (Player3 <> nil) and (Player4 <> nil) then
    AddScore([Player1.Score, Player2.Score, Player3.Score, Player4.Score])
  else
  if (Player1 <> nil) and (Player2 <> nil) and (Player3 <> nil) then
    AddScore([Player1.Score, Player2.Score, Player3.Score])
  else
  if (Player1 <> nil) and (Player2 <> nil) then
    AddScore([Player1.Score, Player2.Score])
  else
  if (Player1 <> nil) then
    AddScore([Player1.Score]);
end;

end.
