{*******************************************************}
{                                                       }
{            Avrora Card Gaming Components              }
{                                                       }
{            Copyright (c) 1999 Avrora LLC              }
{                                                       }
{*******************************************************}
{ This is PROPRIETARY SOURCE CODE of Avrora LLC;        }
{ the contents of this file may not be disclosed to     }
{ third parties, copied or duplicated in any form, in   }
{ whole or in part, without the prior written           }
{ permission of Avrora LLC.                             }
{*******************************************************}

unit ESGraphics;

interface

uses
  Windows, Messages, Classes, Graphics, Controls, Forms;

const
  ESM_EXECPROC       = WM_APP + $0001;
  ESM_INVALIDREGION  = WM_APP + $0002;

type

  { MESSAGE RECORDS }

  TESInvalidateMessage = record
    Msg : Cardinal;
    InvalidRegion : TRect;
    Sender : TObject;
  end;

  { CLASSES } {----------------------------------------------------------------}

  TESCustomGraphicControl = class (TGraphicControl) //(TCustomControl)
  private
    FEmpty: Boolean;
    FPaintBitmap: TBitmap;
    FSyncDraw: Boolean;
    FUpdateCount : Integer;
    procedure ESMInvalidRegion(var Message); message ESM_INVALIDREGION;
    procedure SetEmpty(const Value: Boolean);
  protected
    procedure RepaintCanvas(PaintAll : Boolean); virtual; abstract;
    procedure Paint; override;
    procedure Resize; override;
    procedure PaintBitmapChange(Sender : TObject);
    property PaintBitmap : TBitmap read FPaintBitmap;
    property Empty : Boolean read FEmpty write SetEmpty default True;
    property UpdateCount : Integer read FUpdateCount;
    procedure Loaded; override;
    procedure AdjustSize; override;
    property SyncDraw : Boolean read FSyncDraw write FSyncDraw default True;
  public
    constructor Create(AOwner : TComponent); override;
    destructor Destroy; override;
    procedure ReDraw;
    procedure Clear;
    procedure BeginUpdate;
    procedure EndUpdate;
  end;

  {----------------------------------------------------------------------------}

  TESCustomSprite = class(TComponent)
  private
    FEnabled: Boolean;
    FStopMoving: Boolean;
    FVisible: Boolean;
    FBusy: Boolean;
    FLockCount : Integer;
    FAfterMove: TNotifyEvent;
    procedure SetEnabled(const Value: Boolean);
  protected
    procedure Lock;
    procedure Unlock;
    procedure SetVisible(const Value: Boolean); virtual;
    property StopMoving : Boolean read FStopMoving write FStopMoving;
  public
    constructor Create(AOwner : TComponent); override;
    destructor Destroy; override;
    property Busy : Boolean read FBusy;
    procedure Move(AX, AY, ADestX, ADestY : Integer; ABitmap : TBitmap; Retain : Boolean = False); virtual; abstract;
    procedure MoveBack; virtual; abstract;
    procedure Stop;
  published
    property AfterMove : TNotifyEvent read FAfterMove write FAfterMove;
    property Enabled : Boolean read FEnabled write SetEnabled default True;
    property Visible : Boolean read FVisible write SetVisible stored False;
  end;

{----------------------------------------------------------------------------}

  TESSimpleSprite = class(TESCustomSprite)
  private
    FBitmap: TBitmap;
    FCaption : String;
    FCanvasOwner: TForm;
    FOriginX, FOriginY : Integer;
    FY: Integer;
    FX: Integer;
    FTempBitmap: TBitmap;
    FIncrementY: Integer;
    FIncrementX: Integer;
    FProcessMessages: Boolean;
    FAutoRefresh: Boolean;
    FOnCanvasOwnerPaint: TNotifyEvent;
    FAutoErase: Boolean;
    FSyncDraw: Boolean;
    FDestY: Integer;
    FDestX: Integer;
    procedure SetX(const Value: Integer);
    procedure SetY(const Value: Integer);
    procedure SetIncrementX(const Value: Integer);
    procedure SetIncrementY(const Value: Integer);
    procedure SetProcessMessages(const Value: Boolean);
    procedure SetBitmap(const Value: TBitmap);
  protected
    procedure SetVisible(const Value: Boolean); override;
    procedure Loaded; override;
    procedure Draw(All : Boolean);
    function Erase(DeltaX, DeltaY : Integer) : Boolean;
    procedure DoMove;
    procedure Refresh;
    procedure CanvasOwnerOnPaint(Sender : TObject);
  public
    constructor Create(AOwner : TComponent); override;
    destructor Destroy; override;
    procedure Move(AX, AY, ADestX, ADestY : Integer; ABitmap : TBitmap; Retain : Boolean = False); override;
    procedure MoveBack; override;
    procedure MoveTo;
    property SyncDraw : Boolean read FSyncDraw write FSyncDraw default True;
  published
    property OnCanvasOwnerPaint : TNotifyEvent read FOnCanvasOwnerPaint write FOnCanvasOwnerPaint;
    property CanvasOwner : TForm read FCanvasOwner write FCanvasOwner;
    property X : Integer read FX write SetX default 0;
    property Y : Integer read FY write SetY default 0;
    property IncrementX : Integer read FIncrementX write SetIncrementX default 4;
    property IncrementY : Integer read FIncrementY write SetIncrementY default 4;
    property ProcessMessages : Boolean read FProcessMessages write SetProcessMessages default True;
    property AutoRefresh : Boolean read FAutoRefresh write FAutoRefresh default False;
    property AutoErase : Boolean read FAutoErase write FAutoErase default True;
    property Bitmap : TBitmap read FBitmap write SetBitmap default nil;
    property DestX : Integer read FDestX write FDestX default 0;
    property DestY : Integer read FDestY write FDestY default 0;
  end;

  {----------------------------------------------------------------------------}

procedure DrawCardFrame(ABitmap : TBitmap;   BGColor : TColor);


implementation

uses
  SysUtils;

procedure DrawCardFrame(ABitmap : TBitmap;   BGColor : TColor);
begin
  with ABitmap.Canvas do
  begin
    PolyLine([Point(2,0),Point(68,0),Point(70,2), Point(70,93),
      Point(68,95), Point(2,95), Point(0,93), Point(0,2), Point(2,0)]);
    Pixels[0,0]  := BGColor; Pixels[1,0]  := BGColor; Pixels[0,1]  := BGColor;
    Pixels[69,0] := BGColor; Pixels[70,0] := BGColor; Pixels[70,1] := BGColor;
    Pixels[0,94] := BGColor; Pixels[0,95] := BGColor; Pixels[1,95] := BGColor;
    Pixels[69,95]:= BGColor; Pixels[70,95]:= BGColor; Pixels[70,94]:= BGColor;
  end;
end;


{ TESCustomGraphicControl }

procedure TESCustomGraphicControl.AdjustSize;
begin
  if not (csLoading in ComponentState) then
  if AutoSize then
    SetBounds(Left, Top, FPaintBitmap.Width, FPaintBitmap.Height)
  else
    inherited;
end;

procedure TESCustomGraphicControl.BeginUpdate;
begin
  Inc(FUpdateCount);
end;

procedure TESCustomGraphicControl.Clear;
begin
  FPaintBitmap.Canvas.FillRect(Rect(0,0,PaintBitmap.Width, PaintBitmap.Height));
end;

constructor TESCustomGraphicControl.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FSyncDraw := True;
  FPaintBitmap := TBitmap.Create;
  Canvas.Brush.Color := TForm(Owner).Color;
  FPaintBitmap.Canvas.Brush.Color := Canvas.Brush.Color;
  FEmpty := True;
  ParentColor := True;
end;

destructor TESCustomGraphicControl.Destroy;
begin
  FPaintBitmap.Free;
  inherited Destroy;
end;

procedure TESCustomGraphicControl.EndUpdate;
begin
  Dec(FUpdateCount);
end;

procedure TESCustomGraphicControl.ESMInvalidRegion(var Message);
var
  InvRect : TRect;
begin
  if FUpdateCount = 0 then
  begin
  if Visible then
  begin
    InvRect := TESInvalidateMessage(Message).InvalidRegion;
    { THIS COMMAND DOES THE SAME THING AS TTHREAD.SYNCHRONIZE }
    if FSyncDraw and (not (csDesigning in ComponentState)) then
      SendMessage(TWinControl(Owner).Handle, ESM_EXECPROC, 0, 0);
    try
      if not Empty then
        Canvas.CopyRect(InvRect, PaintBitmap.Canvas, InvRect);
      if Width > PaintBitmap.Width then
        Canvas.FillRect(Rect(PaintBitmap.Width, 0, Width, Height));
      if Height > PaintBitmap.Height then
        Canvas.FillRect(Rect(0, PaintBitmap.Height, Width, Height));
    except

    end;
  end;
  end;
end;

procedure TESCustomGraphicControl.Loaded;
begin
  inherited;
  FPaintBitmap.OnChange := PaintBitmapChange;
end;

procedure TESCustomGraphicControl.Paint;
begin
  if FUpdateCount = 0 then
  begin
    if not Empty then
      Canvas.CopyRect(Canvas.ClipRect, PaintBitmap.Canvas, Canvas.ClipRect)
    else
    if csDesigning in ComponentState then
      Canvas.PolyLine([Point(0, 0), Point(Width-1, 0), Point(Width-1, Height-1), Point(0, Height-1), Point(0,0)]);
  end;
end;

procedure TESCustomGraphicControl.PaintBitmapChange(Sender: TObject);
begin
  if AutoSize then
    SetBounds(Left, Top, TBitmap(Sender).Width, TBitmap(Sender).Height)
  else
  begin
    if FSyncDraw and (not (csDesigning in ComponentState)) then
      SendMessage(TWinControl(Owner).Handle, ESM_EXECPROC, 0, 0);
    if Height > TBitmap(Sender).Height then
      Canvas.FillRect(Rect(0, TBitmap(Sender).Height, Width, Height));
    if Width > TBitmap(Sender).Width then
      Canvas.FillRect(Rect(TBitmap(Sender).Width, 0, Width, Height));
  end;
end;

procedure TESCustomGraphicControl.ReDraw;
begin
  if FSyncDraw and (not (csDesigning in ComponentState)) then
    SendMessage(TWinControl(Owner).Handle, ESM_EXECPROC, 0, 0);
  Canvas.Draw(0,0,FPaintBitmap);
end;

procedure TESCustomGraphicControl.Resize;
begin
  inherited;
  if AutoSize then
    SetBounds(Left, Top, FPaintBitmap.Width, FPaintBitmap.Height);
end;

procedure TESCustomGraphicControl.SetEmpty(const Value: Boolean);
begin
  FEmpty := Value;
  Invalidate;
end;

{ TESCustomSprite }

constructor TESCustomSprite.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FEnabled := True;
  FLockCount := 0;
end;

destructor TESCustomSprite.Destroy;
begin
  inherited Destroy;
end;

procedure TESCustomSprite.SetEnabled(const Value: Boolean);
begin
  FEnabled := Value;
  if not Value then
    FStopMoving := True;
end;

procedure TESCustomSprite.SetVisible(const Value: Boolean);
begin
  FVisible := Value;
end;

procedure TESCustomSprite.Lock;
begin
  if FLockCount = 0 then
    FBusy := True;
  Inc(FLockCount);
end;

procedure TESCustomSprite.Unlock;
begin
  Dec(FLockCount);
  if FLockCount = 0 then
    FBusy := False;
end;

procedure TESCustomSprite.Stop;
begin
  FStopMoving := True;
  Visible := False;
  Application.ProcessMessages;
end;

{ TESSimpleSprite }

procedure TESSimpleSprite.SetBitmap(const Value: TBitmap);
begin
  FBitmap.Assign(Value);
end;

procedure TESSimpleSprite.CanvasOwnerOnPaint(Sender: TObject);
begin
  if FAutoRefresh then
    Refresh;
  if Assigned(FOnCanvasOwnerPaint) then
    FOnCanvasOwnerPaint(FCanvasOwner);
end;

constructor TESSimpleSprite.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FSyncDraw := True;
  FBitmap := TBitmap.Create;
  FTempBitmap := TBitmap.Create;;
  if AOwner is TForm then
  begin
    FCanvasOwner := TForm(AOwner);
    FCaption := FCanvasOwner.Caption;
  end;
  FIncrementX := 4;
  FIncrementY := 4;
  FAutoErase := True;
  FProcessMessages := True;
end;

destructor TESSimpleSprite.Destroy;
begin
  FBitmap.Free;
  FTempBitmap.Free;
  inherited Destroy;
end;

procedure TESSimpleSprite.DoMove;
var
  SX, SY : Integer;
begin
  Lock;
  try
    SX := FIncrementX;
    SY := FIncrementY;
    StopMoving := False;
    repeat
      if FProcessMessages then
        Application.ProcessMessages;
      if StopMoving or Application.Terminated then
        Abort;
      if X > DestX then
      begin
        if X - SX < DestX then
           X := DestX
       else
         X := X - SX;
      end
      else
      if X < DestX then
      begin
        if X + SX > DestX then
          X := DestX
        else
          X := X + SX;
      end;
      if Y > DestY then
      begin
        if Y - SY < DestY then
          Y := DestY
        else
        Y := Y - SY;
      end
      else
      if Y < DestY then
      begin
        if Y + SY > DestY then
          Y := DestY
        else
        Y := Y + SY;
      end;
    until ((X = DestX) and (Y = DestY));
  finally
    Unlock;
    if Assigned(AfterMove) then
      AfterMove(Self);
  end;
end;

procedure TESSimpleSprite.Draw(All : Boolean);
begin
  { THIS COMMAND DOES THE SAME THING AS TTHREAD.SYNCHRONIZE OR I GUESS LIKE THAT}
  if FSyncDraw and (not (csDesigning in ComponentState)) then
    SendMessage(TWinControl(Owner).Handle, ESM_EXECPROC, 0, 0);
  try
    if All then
    begin
      FTempBitmap.Width := FBitmap.Width;
      FTempBitmap.Height := FBitmap.Height;
      FTempBitmap.Canvas.CopyRect(Rect(0, 0, Bitmap.Width, Bitmap.Height),
        FCanvasOwner.Canvas, Rect(FX, FY, FX+Bitmap.Width, FY+Bitmap.Height));
    end;
      FCanvasOwner.Canvas.Draw(FX, FY, FBitmap);
  except
  end;
end;

function TESSimpleSprite.Erase(DeltaX, DeltaY : Integer) : Boolean;
var
  W, H : Integer;
begin
  Result := True;
  W := FTempBitmap.Width;
  H := FTempBitmap.Height;
  if ((Abs(DeltaX)>=W) or (ABS(DeltaY)>=H)) then
    FCanvasOwner.Canvas.Draw(FX, FY, FTempBitmap)
  else
  if ((DeltaX <> 0) or (DeltaY <> 0)) then
  begin
    Result := False;
    if DeltaX > 0 then
    begin
      FCanvasOwner.Canvas.CopyRect(Rect(FX, FY, FX+DeltaX, FY+H), FTempBitmap.Canvas, Rect(0,0,DeltaX, H));
      FTempBitmap.Canvas.CopyRect(Rect(0,0,W-DeltaX, H), FTempBitmap.Canvas, Rect(DeltaX, 0, W, H));
      FTempBitmap.Canvas.CopyRect(Rect(W-DeltaX,0,W, H), FCanvasOwner.Canvas, Rect(FX+W, FY, FX+W+DeltaX, FY+H));
    end
    else
    begin
      FCanvasOwner.Canvas.CopyRect(Rect(FX+W+DeltaX, FY, FX+W+DeltaX+Abs(DeltaX),  FY+H), FTempBitmap.Canvas, Rect(W+DeltaX,0, W, H));
      FTempBitmap.Canvas.CopyRect(Rect(Abs(DeltaX), 0,W, H), FTempBitmap.Canvas, Rect(0,0,W+DeltaX, H));
      FTempBitmap.Canvas.CopyRect(Rect(0,0,Abs(DeltaX), H), FCanvasOwner.Canvas, Rect(FX+DeltaX, FY, FX , FY+H));
    end;
    if DeltaY > 0 then
    begin
      FCanvasOwner.Canvas.CopyRect(Rect(FX, FY, FX+W, FY+DeltaY), FTempBitmap.Canvas, Rect(0,0,W, DeltaY));
      FTempBitmap.Canvas.CopyRect(Rect(0,0,W, H-DeltaY), FTempBitmap.Canvas, Rect(0, DeltaY, W, H));
      FTempBitmap.Canvas.CopyRect(Rect(0, H-DeltaY,W, H), FCanvasOwner.Canvas, Rect(FX, FY+H, FX+W, FY+H+DeltaY));
    end
    else
    begin
      FCanvasOwner.Canvas.CopyRect(Rect(FX, FY+H+DeltaY, FX+W  ,  FY+H+DeltaY+Abs(DeltaY)), FTempBitmap.Canvas, Rect(0, H+DeltaY, W, H));
      FTempBitmap.Canvas.CopyRect(Rect(0, Abs(DeltaY), W, H), FTempBitmap.Canvas, Rect(0,0,W, H+DeltaY));
      FTempBitmap.Canvas.CopyRect(Rect(0,0,W, Abs(DeltaY)), FCanvasOwner.Canvas, Rect(FX, FY+DeltaY, FX+W , FY));
    end;
  end;
end;

procedure TESSimpleSprite.Loaded;
begin
  inherited;
  FCanvasOwner.OnPaint := CanvasOwnerOnPaint;
end;

procedure TESSimpleSprite.Move(AX, AY, ADestX, ADestY : Integer; ABitmap : TBitmap; Retain : Boolean = False);
begin
  if Enabled then
  begin
    if ABitmap <> nil then
      Bitmap.Assign(ABitmap);
    Bitmap.Transparent := False;
    FX := AX;
    FY := AY;
    FOriginX := FX;
    FOriginY := FY;
    DestX := ADestX;
    DestY := ADestY;
    Visible := True;
    DoMove;
    if not Retain then
      Visible := False;
  end;
end;

procedure TESSimpleSprite.MoveBack;
begin
  if Enabled then
  begin
    FDestX := FOriginX;
    FDestY := FOriginY;
    MoveTo;
    Visible := False;
  end;
end;

procedure TESSimpleSprite.MoveTo;
begin
  if Enabled then
    DoMove;
end;

procedure TESSimpleSprite.Refresh;
begin
  if FVisible then
    Draw(False);
end;

procedure TESSimpleSprite.SetIncrementX(const Value: Integer);
begin
  if Value >= 1 then
    FIncrementX := Value;
end;

procedure TESSimpleSprite.SetIncrementY(const Value: Integer);
begin
  if Value >= 1 then
    FIncrementY := Value;
end;

procedure TESSimpleSprite.SetProcessMessages(const Value: Boolean);
begin
  FProcessMessages := Value;
end;

procedure TESSimpleSprite.SetVisible(const Value: Boolean);
begin
  inherited;
  if Value then
    Draw(True)
  else
  begin
    Erase(FTempBitmap.Width + 100, 0);
    FTempBitmap.Width := 0;
    FTempBitmap.Height := 0;
  end;
end;

procedure TESSimpleSprite.SetX(const Value: Integer);
var
  All : Boolean;
begin
  All := True;
  if FVisible and FAutoErase then
    All := Erase(Value - FX, 0);
  FX := Value;
  if FVisible then
    Draw(All);
end;

procedure TESSimpleSprite.SetY(const Value: Integer);
var
  All : Boolean;
begin
  All := True;
  if FVisible and FAutoErase then
    All := Erase(0, Value - FY);
  FY := Value;
  if FVisible then
    Draw(All);
end;

end.
